from enum import Enum


class Color(Enum):
    COLOR_1 = 'color_1'
    DEFAULT = None


class Align(Enum):
    LEFT = 'left'
    CENTER = 'center'
    RIGHT = 'right'


class Mode(Enum):
    MONO = 'mono'


class Font(Enum):
    A = 'font_a'
    B = 'font_b'


class BarcodeType(Enum):
    UPC_A = 'upc_a'
    UPC_E = 'upc_e'
    EAN13 = 'ean13'
    JAN13 = 'jan13'
    EAN8 = 'ean8'
    JAN8 = 'jan8'
    CODE39 = 'code39'
    ITF = 'itf'
    CODABAR = 'codabar'
    CODE93 = 'code93'
    CODE128 = 'code128'
    GS1_128 = 'gs1_128'
    GS1_DATABAR_OMNIDIRECTIONAL = 'gs1_databar_omnidirectiona'
    GS1_DATABAR_TRUNCATED = 'gs1_databar_truncated'
    GS1_DATABAR_LIMITED = 'gs1_databar_limited'
    GS1_DATABAR_EXPANDED = 'gs1_databar_expanded'


class HRI(Enum):
    NONE = 'None'
    ABOVE = 'above'
    BELOW = 'below'
    BOTH = 'both'


class CutType(Enum):
    NO_FEED = 'no_feed'
    FEED = 'feed'


class Lang(Enum):
    EN = 'en'
    DE = 'de'
    FR = 'fr'
    IT = 'it'
    ES = 'es'



import requests

from .constants import BarcodeType
from .document import EposDocument
from .elements import Text, Feed, Cut, Barcode


class Printer:
    def __init__(self, ip):
        self.ip = ip

        # Defaults, normally not changed
        self.use_https = False
        self.devid = 'local_printer'
        self.timeout = 5000
        self.url = '/cgi-bin/epos/service.cgi'

    def try_connection(self):
        """Send empty page and check the status"""
        doc = EposDocument()
        resp = self._send_printjob(doc.body_to_str())
        return 'success="true' in resp

    def print(self, doc: EposDocument, autocut=True):
        if autocut:
            doc.add_body(Cut())
        resp = self._send_printjob(doc.body_to_str())
        return resp

    def _send_printjob(self, data):
        prefix = 'https://' if self.use_https else 'http://'
        url = prefix + self.ip + self.url
        headers = {
            'content-type': 'text/xml; charset=utf-8',
            'If-Modified-Since': 'Thu, 01 Jan 1970 00:00:00 GMT',
            'SOAPAction': '""',
        }
        params = {'devid': self.devid, 'timeout': self.timeout}

        #print(data)

        response = requests.post(
            url,
            data=_add_soap_enveloppe(data),
            headers=headers,
            params=params,
        )

        return response.text


def _add_soap_enveloppe(body: str, header: str = '') -> str:
    """Add the soap enveloppe, header and body"""
    soap = []

    soap.append('<s:Envelope xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">')
    if header:
        soap.append('<s:Header>')
        soap.append(header)
        soap.append('/<s:Header>')
    soap.append('<s:Body>')
    soap.append(body)
    soap.append('</s:Body>')
    soap.append('</s:Envelope>')

    return ''.join(soap)



from dataclasses import dataclass, field
from typing import Type, Generic, TypeVar
import xml.etree.ElementTree as ET

from .elements import BaseElement

B = TypeVar('B', bound='BaseElement')


@dataclass
class EposDocument:
    parameters: list = field(default_factory=list)
    body: list[Type[BaseElement]] = field(default_factory=list)

    def add_parameter(self, element: Generic[B]):
        self.parameters.append(element)

    def add_body(self, element: Generic[B]):
        self.body.append(element)

    def parameters_to_xml(self) -> ET.Element:
        return _to_xml('parameter', self.parameters)

    def body_to_xml(self) -> ET.Element:
        return _to_xml('epos-print', self.body)

    def parameters_to_str(self) -> str:
        return ET.tostring(self.parameters_to_xml(), encoding='unicode', short_empty_elements=False)

    def body_to_str(self, url_encode_newlines=False) -> str:
        s = ET.tostring(self.body_to_xml(), encoding='unicode')
        s = s.replace(' />', '/>')
        if url_encode_newlines:
            s = s.replace('\n', '&#10;')
        return s


def _to_xml(base_tag: str, element_list: list[Type[BaseElement], ...]) -> ET.Element:
    root = ET.Element(
        base_tag,
        xmlns='http://www.epson-pos.com/schemas/2011/03/epos-print',
    )
    for element in element_list:
        root.append(element.to_xml())
    return root



from .constants import Align, Font, Color


class AlignAtt:
    def __init__(self, align: Align, **kwargs):
        super().__init__(**kwargs)
        self.align = align

    @property
    def align(self) -> Align:
        return self._align

    @align.setter
    def align(self, align: Align):
        try:
            align = Align(align)
        except ValueError:
            pass

        if align is not None and align not in Align:
            raise ValueError('Unknown alignment')
        self._align = align


class WidthAtt:
    def __init__(self, width: int, min_width: int, max_width: int, **kwargs):
        super().__init__(**kwargs)
        self._min_width = min_width
        self._max_width = max_width
        self.width = width

    @property
    def width(self) -> int:
        return self._width

    @width.setter
    def width(self, width: int):
        try:
            width = int(width)
        except TypeError:
            pass

        if width is not None and (width < self._min_width or width > self._max_width):
            raise ValueError(
                f'"width" must be bewteen {self._min_width} and {self._max_width} inclusive')
        self._width = width


class HeightAtt:
    def __init__(self, height: int, min_height: int, max_height: int, **kwargs):
        super().__init__(**kwargs)
        self._min_height = min_height
        self._max_height = max_height
        self.height = height

    @property
    def height(self) -> int:
        return self._height

    @height.setter
    def height(self, height: int):
        try:
            height = int(height)
        except TypeError:
            pass

        if height is not None and (height < self._min_height or height > self._max_height):
            raise ValueError(
                f'"height" must be bewteen {self._min_height} and {self._max_height} inclusive')
        self._height = height


class FontAtt:
    def __init__(self, font: Font, **kwargs):
        super().__init__(**kwargs)
        self.font = font

    @property
    def font(self) -> Font:
        return self._font

    @font.setter
    def font(self, font: Font):
        try:
            font = Font(font)
        except ValueError:
            pass

        if font is not None and font not in Font:
            raise ValueError(
                f'"font" must be valid.')
        self._font = font


class LineSpcAtt:
    def __init__(self, linespc, min_linespc=0, max_linespc=255, **kwargs):
        super().__init__(**kwargs)
        self._min_linespc = min_linespc
        self._max_linespc = max_linespc
        self.linespc = linespc

    @property
    def linespc(self) -> int:
        return self._linespc

    @linespc.setter
    def linespc(self, linespc: int):
        try:
            linespc = int(linespc)
        except TypeError:
            pass

        if linespc is not None and (self._min_linespc > linespc or linespc > self._max_linespc):
            raise ValueError(
                f'"linespc" must be between {self._min_linespc} and {self._max_linespc} inclusive'
            ) from None
        self._linespc = linespc


class RotateAtt:
    def __init__(self, rotate, **kwargs):
        super().__init__(**kwargs)
        self._rotate = rotate

    @property
    def rotate(self) -> bool:
        return self._rotate

    @rotate.setter
    def rotate(self, rotate: bool):
        try:
            rotate = bool(rotate)
        except TypeError:
            pass

        self._rotate = rotate


class ColorAtt:
    def __init__(self, color, **kwargs):
        super().__init__(**kwargs)
        self.color = color

    @property
    def color(self) -> Color:
        return self._color

    @color.setter
    def color(self, color: Color):
        try:
            color = Color(color)
        except TypeError:
            pass

        if color not in Color:
            raise ValueError('Unknonw color')
        self._color = color



import xml.etree.ElementTree as ET
from typing import List
from dataclasses import dataclass

from . import status
from .attributes import AlignAtt, ColorAtt, WidthAtt, HeightAtt, FontAtt, RotateAtt, LineSpcAtt
from .constants import Color, Align, Mode, BarcodeType, HRI, Font, CutType, Lang

@dataclass
class BaseElement:
    """
    Base class for all ePOS-Print XML elements
    """
    tag: str
    text: str

    def __init__(self, tag, text='', **kwargs):
        super().__init__(**kwargs)
        self.tag = tag
        self.attr = {}
        self.text = text
        self.tail = ''

        self.namespaces = {
            's': 'http://schemas.xmlsoap.org/soap/envelope/',
            'epos-print': 'http://www.epson-pos.com/schemas/2011/03/epos-print',
        }

    def __repr__(self):
        return f'{self.tag.upper()}: {repr(self.text)} {self.attr}'

    def to_xml(self):
        """
        Converts the object to XML
        """
        self.attr = {}
        self._load_attrs()
        attr = {k: str(v).lower() for k, v in self.attr.items() if v is not None}
        element = ET.Element(self.tag, attr)
        element.text = self.text
        element.tail = self.tail
        return element

    def to_str(self):
        return ET.tostring(self.to_xml(), encoding='utf-8', )

    def from_xml(self, xml: str):
        """
        Loads xml data
        """
        dom = ET.fromstring(xml)

    def _add_ns(self, tag: str) -> str:
        r = f'{{{self.namespaces["a"]}}}{tag}'
        # print(r)
        return r

    def _load_attrs(self):
        """To be overwritten by element class"""
        raise NotImplementedError


class Response(BaseElement):
    """
    This is an XML document that is sent back from a printer to an application.
    Reference:
    https://reference.epson-biz.com/modules/ref_epos_print_xml_en/index.php?vid=ref_epos_print_xml_en_xmlforcontrollingprinter_response
    """
    def __init__(self, success=True):
        super().__init__('response')

        self.success = success
        self.code = ''
        self.status = 0
        self.battery = 0

    def _load_attrs(self):
        self.attr['xmlns'] = self.namespaces['epos-print']
        self.attr['success'] = str(self.success).lower()
        if self.code:
            self.attr['code'] = self.code
        self.attr['status'] = str(self.status)
        self.attr['battery'] = str(self.battery)

    def get_status(self) -> List[str]:
        return status.parse_code(self.status)


class Text(BaseElement, FontAtt, WidthAtt, HeightAtt, AlignAtt, LineSpcAtt, RotateAtt, ColorAtt):
    def __init__(
            self,
            text: str = '',
            lang: Lang = None,
            font: Font = None,
            smooth: bool = None,
            double_width: bool = None,
            double_height: bool = None,
            width: int = None,
            height: int = None,
            reverse: bool = None,
            underline: bool = None,
            bold: bool = None,
            color: Color = None,
            x: int = None,
            y: int = None,
            align: Align = None,
            rotate: bool = None,
            linespc: int = None
    ):
        super().__init__(
            tag='text',
            text=text,
            font=font,
            width=width, min_width=1, max_width=8,
            height=height, min_height=1, max_height=8,
            align=align,
            linespc=linespc,
            rotate=rotate,
            color=color
        )

        self.lang = lang
        self.smooth = smooth
        self.double_width = double_width
        self.double_height = double_height
        self.reverse = reverse
        self.underline = underline
        self.bold = bold
        self.x = x
        self.y = y

    def _load_attrs(self):
        self.attr['lang'] = self.lang.value if self.lang else None
        self.attr['font'] = self.font.value if self.font else None
        self.attr['dw'] = self.double_width
        self.attr['dh'] = self.double_height
        self.attr['width'] = self.width
        self.attr['height'] = self.height
        self.attr['reverse'] = self.reverse
        self.attr['em'] = self.bold
        self.attr['color'] = self.color.value if self.color else None
        self.attr['x'] = self.x
        self.attr['y'] = self.y
        self.attr['align'] = self.align.value if self.align else None
        self.attr['rotate'] = self.rotate
        self.attr['linespc'] = self.linespc
        self.attr['smooth'] = self.smooth
        self.attr['ul'] = self.underline

    @property
    def lang(self) -> Lang:
        return self._lang

    @lang.setter
    def lang(self, lang: Lang):
        try:
            lang = Lang(lang)
        except ValueError:
            pass

        if lang is not None and lang not in Lang:
            raise ValueError('"lang" is invalid') from None
        self._lang = lang

    @property
    def smooth(self) -> bool:
        return self._smooth

    @smooth.setter
    def smooth(self, smooth: bool):
        if smooth == 'true':
            self._smooth = True
        elif smooth == 'false':
            self._smooth = False
        else:
            self._smooth = smooth
    
    @property
    def double_width(self) -> bool:
        return self._double_width

    @double_width.setter
    def double_width(self, double_width: bool):
        if double_width == 'true':
            self._double_width = True
        elif double_width == 'false':
            self._double_width = False
        else:
            self._double_width = double_width

    @property
    def dw(self):
        return self.double_width

    @dw.setter
    def dw(self, dw):
        self.double_width = dw
    
    @property
    def double_height(self) -> bool:
        return self._double_height

    @double_height.setter
    def double_height(self, double_height: bool):
        if double_height == 'true':
            self._double_height = True
        elif double_height == 'false':
            self._double_height = False
        else:
            self._double_height = double_height

    @property
    def dh(self):
        return self.double_height

    @dh.setter
    def dh(self, dh):
        self.double_height = dh
    
    @property
    def reverse(self) -> bool:
        return self._reverse

    @reverse.setter
    def reverse(self, reverse: bool):
        if reverse == 'true':
            self._reverse = True
        elif reverse == 'false':
            self._reverse = False
        else:
            self._reverse = reverse

    @property
    def underline(self) -> bool:
        return self._underline

    @underline.setter
    def underline(self, underline: bool):
        if underline == 'true':
            self._underline = True
        elif underline == 'false':
            self._underline = False
        else:
            self._underline = underline
    
    @property
    def ul(self):
        return self.underline
    
    @ul.setter
    def ul(self, ul):
        self.underline = ul
    
    @property
    def bold(self) -> bool:
        return self._bold

    @bold.setter
    def bold(self, bold: bool|str):
        if bold == 'true':
            self._bold = True
        elif bold == 'false':
            self._bold = False
        else:
            self._bold = bold

    @property
    def em(self):
        return self.bold

    @em.setter
    def em(self, em):
        self.bold = em
    
    @property
    def x(self) -> int:
        return self._x

    @x.setter
    def x(self, x: int):
        try:
            x = int(x)
        except TypeError:
            pass
        
        if x is not None and (x < 0 or x > 576):
            raise ValueError('"x" must be between 0 and 576 inclusive') from None
        self._x = x

    @property
    def y(self) -> int:
        return self._y

    @y.setter
    def y(self, y: int):
        try:
            y = int(y)
        except TypeError:
            pass

        if y is not None and (y < 0 or y > 65535):
            raise ValueError('"y" must be between 0 and 576 inclusive') from None
        self._y = y


class Feed(BaseElement, LineSpcAtt):
    def __init__(
            self,
            line: int = None,
            unit: int = None,
            linespc: int = None
    ):
        super().__init__(
            tag='feed',
            linespc=linespc
        )
        self.unit = unit
        self.line = line

    def _load_attrs(self):
        self.attr['unit'] = self.unit
        self.attr['line'] = self.line
        self.attr['linespc'] = self.linespc

    @property
    def unit(self) -> int:
        return self._unit

    @unit.setter
    def unit(self, unit: int):
        try:
            unit = int(unit)
        except TypeError:
            pass

        if unit is not None and (0 > unit or unit > 255):
            raise ValueError('"unit" must be between 0 and 255 inclusive') from None
        self._unit = unit

    @property
    def line(self) -> int:
        return self._line

    @line.setter
    def line(self, line: int):
        try:
            line = int(line)
        except TypeError:
            pass

        if line is not None and (line < 0 or line > 255):
            raise ValueError('"line" must be between 0 and 255 inclusive') from None
        self._line = line


class Image(BaseElement, AlignAtt, ColorAtt, WidthAtt, HeightAtt):
    def __init__(
            self,
            width: int = 0, height: int = 0,
            text: str = '',
            color: Color = None,
            align: Align = None,
            mode: Mode = Mode.MONO
    ):
        super().__init__(
            tag='image',
            text=text,
            align=align,
            width=width, min_width=0, max_width=576,
            height=height, min_height=0, max_height=655635,
            color=color
        )
        self.mode = mode

    def _load_attrs(self):
        self.attr['width'] = self.width
        self.attr['height'] = self.height
        self.attr['color'] = self.color.value
        self.attr['align'] = self.align.value if self.align else None
        if self.mode != Mode.MONO:
            self.attr['mode'] = self.mode.value

    @property
    def mode(self) -> Mode:
        return self._mode

    @mode.setter
    def mode(self, mode: Mode):
        try:
            mode = Mode(mode)
        except TypeError:
            pass

        if mode not in Mode:
            raise ValueError('Unknown mode')
        self._mode = mode


class Logo(BaseElement, AlignAtt):
    def __init__(
            self,
            key1: int = 0,
            key2: int = 0,
            align: Align = None
    ):
        super().__init__(tag='logo', align=align)
        self.key1 = key1
        self.key2 = key2

    def _load_attrs(self):
        self.attr['key1'] = str(self.key1)
        self.attr['key2'] = str(self.key2)
        self.attr['align'] = self.align
    
    @property
    def key1(self) -> int:
        return self._key1
    
    @key1.setter
    def key1(self, key1: int):
        try:
            key1 = int(key1)
        except TypeError:
            pass

        if key1 < 0 or key1 > 255:
            raise ValueError('"key1" must be bewteen 0 and 255 inclusive') from None
        self._key1 = key1

    @property
    def key2(self) -> int:
        return self._key2

    @key2.setter
    def key2(self, key2: int):
        try:
            key2 = int(key2)
        except TypeError:
            pass

        if key2 < 0 or key2 > 255:
            raise ValueError('"key2" must be bewteen 0 and 255 inclusive') from None
        self._key2 = key2


class Barcode(BaseElement, AlignAtt, WidthAtt, HeightAtt, FontAtt, RotateAtt):
    def __init__(
            self,
            type: BarcodeType = BarcodeType,
            text='',
            hri: HRI = None,
            font: Font = None,
            width: int = None,
            height: int = None,
            align: Align = None,
            rotate: bool = None
    ):
        super().__init__(
            tag='barcode',
            text=text,
            font=font,
            width=width, min_width=2, max_width=6,
            height=height, min_height=0, max_height=255,
            align=align,
            rotate=rotate
        )
        self.type = type
        self.hri = hri

    def _load_attrs(self):
        self.attr['type'] = self.type.value
        self.attr['hri'] = self.hri.value if self.hri else None
        self.attr['font'] = self.font.value if self.font else None
        self.attr['width'] = self.width
        self.attr['height'] = self.height
        self.attr['align'] = self.align.value if self.align else None
        self.attr['rotate'] = self.rotate

    @property
    def type(self) -> BarcodeType:
        return self._type

    @type.setter
    def type(self, type: BarcodeType):
        try:
            type = BarcodeType(type)
        except TypeError:
            pass

        if type not in BarcodeType:
            raise ValueError('The barcodetype is invalid') from None
        self._type = type

    @property
    def hri(self) -> HRI:
        return self._hri

    @hri.setter
    def hri(self, hri: HRI):
        try:
            hri = HRI(hri)
        except ValueError:
            pass

        if hri is not None and hri not in HRI:
            raise ValueError('The HRI is invalid') from None
        self._hri = hri


class Symbol(BaseElement):  # TODO implement
    def __init__(self):
        super().__init__('')
        raise NotImplementedError()


class HLine(BaseElement):
    def __init__(self):
        super().__init__('')
        raise NotImplementedError()


class VLineBegin(BaseElement):
    def __init__(self):
        super().__init__('')
        raise NotImplementedError()


class VLineEnd(BaseElement):
    def __init__(self):
        super().__init__('')
        raise NotImplementedError()


class Page(BaseElement):  # TODO implement
    def __init__(self):
        super().__init__('')
        raise NotImplementedError()


class Area(BaseElement):  # TODO implement
    def __init__(self):
        super().__init__('')
        raise NotImplementedError()


class Direction(BaseElement):  # TODO implement
    def __init__(self):
        super().__init__('')
        raise NotImplementedError()


class Position(BaseElement):  # TODO implement
    def __init__(self):
        super().__init__('')
        raise NotImplementedError()


class Line(BaseElement):
    def __init__(self):
        super().__init__('')
        raise NotImplementedError()


class Rectangle(BaseElement):  # TODO implement
    def __init__(self):
        super().__init__('')
        raise NotImplementedError()


class Cut(BaseElement):
    def __init__(self, type: CutType = None):
        super().__init__('cut')
        self.type = type

    def _load_attrs(self):
        self.attr['type'] = self.type
        
    @property
    def type(self) -> CutType:
        return self._type

    @type.setter
    def type(self, type: CutType):
        try:
            type = CutType(type)
        except ValueError:
            pass

        if type is not None and type not in CutType:
            raise ValueError('Unknown type')
        self._type = type


class Pulse(BaseElement):  # TODO implement
    def __init__(self):
        super().__init__('')
        raise NotImplementedError()


class Sound(BaseElement):  # TODO implement
    def __init__(self):
        super().__init__('')
        raise NotImplementedError()


class Command(BaseElement):  # TODO implement
    def __init__(self):
        super().__init__('')
        raise NotImplementedError()


class Layout(BaseElement):
    def __init__(self):
        super().__init__('')
        raise NotImplementedError()


class Recovery(BaseElement):
    def __init__(self):
        super().__init__('recovery')

    def _load_attrs(self):
        pass


class Reset(BaseElement):
    def __init__(self):
        super().__init__('reset')

    def _load_attrs(self):
        pass


class BatchBegin(BaseElement):
    def __init__(self):
        super().__init__('')
        raise NotImplementedError()


class BatchEnd(BaseElement):
    def __init__(self):
        super().__init__('')
        raise NotImplementedError()


class RotateBegin(BaseElement):
    def __init__(self):
        super().__init__('')
        raise NotImplementedError()


class RotateEnd(BaseElement):
    def __init__(self):
        super().__init__('')
        raise NotImplementedError()



def parse_code(asb: int) -> [str]:
    msg = []

    if asb & 0x00000001:
        msg.append('No printer response')
    if asb & 0x00000002:
        msg.append('Printing is succssfully completed')
    if asb & 0x00000004:
        msg.append('Status of the drawer kick number 3 connector pin = "H"')
    if asb & 0x00000008:
        msg.append('Offline status')
    if asb & 0x00000020:
        msg.append('Cover is open')
    if asb & 0x00000040:
        msg.append('Paper feed switch is feeding paper')
    if asb & 0x00000100:
        msg.append('Waiting for online recovery')
    if asb & 0x00000200:
        msg.append('Panel switch is ON')
    if asb & 0x00000400:
        msg.append('Mechanical error generated')
    if asb & 0x00000800:
        msg.append('Auto cutter error generated')
    if asb & 0x00002000:
        msg.append('Unrecoverable error generated')
    if asb & 0x00004000:
        msg.append('Auto recovery error generated')
    if asb & 0x00010000:
        msg.append('Waiting for insertion of a slip sheet for slip printing')
    if asb & 0x00020000:
        msg.append('Roll paper has almost run out')
    if asb & 0x00040000:
        msg.append('Waiting for ejection of a slip sheet for slip printing')
    if asb & 0x00080000:
        msg.append('Roll paper has run out')
    if asb & 0x01000000:
        msg.append('Buzzer is sounding OR Waiting for paper removal')
    if asb & 0x80000000:
        msg.append('No paper is detected with the paper removal detector ')
    if asb & 0x80000000:
        msg.append('Spooler stopped')

    return msg



from epos.printer import Printer
from epos.document import EposDocument
from epos.elements import Text, Feed, Barcode
from epos.constants import Align, BarcodeType, HRI

# Create a new printer object with 10.0.0.10 as ip
printer = Printer('10.0.0.10')
# Check if we can connect to the printer with no errors, otherwise exit
if not printer.try_connection():
    exit()
print('Connection with printer established.')

# Create a new EposDocument object
# This will contain all the individual elements
doc = EposDocument()

# Add an element directly to the body of the document
doc.add_body(Text('This is example text!\n'))
doc.add_body(Feed()) # Another way to add a newline

# It's also possible to first create the text object and then change the properties
t = Text('Some special text :O\n')
t.bold = True
t.align = Align.CENTER
t.double_width = False
t.width = 1
doc.add_body(t)

# Rotated text
t = Text('This text is rotated\n')
t.rotate = True
doc.add_body(t)

# Find max length in double mode
t = Text("|"*25)
t.double_width = True
t.double_height = True
t.rotate = False
t.reverse = True
t.align = Align.LEFT
t.bold = False
doc.add_body(t)

# Add 2 empty lines
doc.add_body(Feed(5))

# Add a barcode
doc.add_body(Barcode(BarcodeType.CODE39, '01234567890', align=Align.LEFT, width=2, height=64, hri=HRI.BELOW))

# Send the whole document to the printer
# This will automatially send a Cut at the end of the document body
r = printer.print(doc)
print(r)